// import java.awt.*;
import java.io.* ;
import java.net.* ;
import java.util.* ;

class SchedReader
{
  BART applet;
  String fstr ;	//  train schedule buffer in String format
  int startw ;		//  start of next integer in schedule buffer
  int endw ;			//  end of next integer in schedule buffer
  boolean is_pc ;	//  whether lines ended by \r\n
  int n_blines ;		//  number of lines
  int n_bsegs ;		//  number of segments
  int start_order[];
  Bline blines[] ;		//  line data structure
  byte fbuf[] = new byte[32767] ;	// buffer holding schedule

  SchedReader(BART app)
  {
    applet = app;
    int total = get_schedule(applet.which_schedule);
      // read compressed schedule data
    read_data(total) ;
  }
      // read schedule into one big buffer
  int get_schedule(int which_schedule)
  {
    URL url=null ;
    URLConnection connection=null ;
    DataInputStream url_data=null ;
    int num_read = 0 ;
    int offset = 0 ;
    int len = 1024 ;
    int total = 0 ;

    try { url = new URL(applet.getCodeBase().toString()+"wd_data.html") ; }
    catch(MalformedURLException e) { System.exit(0) ; }
    try { connection = url.openConnection() ; }
    catch(IOException e) { System.exit(0) ; }
      // read data
    try {
      url_data = new DataInputStream(new BufferedInputStream(
                                connection.getInputStream())) ;
    }
    catch(IOException e) { System.exit(0) ; }
      // read the data
    while (num_read != -1)
    {
      try { num_read = url_data.read(fbuf,offset,len) ; }
      catch(IOException e) { System.exit(0) ; }
      if (num_read != -1)
      {
        total += num_read ;
        offset += num_read ;
      }
    }
      // return total bytes read
    return(total) ;
  }

    //  read train schedule buffer
    //  decompresses run-length encoded portion of schedule
  void read_data(int total)
  {
    int i ;
    Bsegment bsegs[] ;		//  segment data structure
	
      //  convert byte buffer into String buffer
      //  ideally check encoding first, but I can't find one that Java
      //  accepts at the moment
    fstr = new String(fbuf) ;
      //  check to see if PC format for end of lines
    startw = fstr.indexOf('\r',0) ;
    if (startw >= 0)
    {
        is_pc = true ;
    }
    else
    {
        is_pc = false ;
    }
    
      //  get position of first integer
    startw = fstr.indexOf('\n',0) + 1 ;
    startw = fstr.indexOf('\n',startw) + 1 ;

      //  number of segments
    n_bsegs = iget() ;
    bsegs = new Bsegment[n_bsegs] ;
      // for all segments; a segment is a unique section of track
      // shared by most lines
    for (i = 0 ; i < n_bsegs ; i++)
      bsegs[i] = new Bsegment(this, i) ;

      //  number of BART lines
    n_blines = iget() ;
    blines = new Bline[n_blines] ;

      // read segment information for all BART lines
    for (i = 0 ; i < n_blines ; i++)
      blines[i] = new Bline(applet.train_size, this, bsegs, i) ;

    start_order = new int[n_blines] ;
      // read starting order for all lines
    for (i = 0 ; i < n_blines ; i++)
      start_order[i] = iget() ;

      // read schedule for all lines
    for (i = 0 ; i < n_blines; i++)
      blines[i].readTimes();
  }


        // line information has been run-length encoded; decode it
  void decode_runs(int foo,int npixels,
                   int info[],int loc,boolean do_add,boolean check_neg)
  {
    int ctr, val, i ;
    int tmp ;

    while (loc < npixels)
    {
      tmp = iget() ;
        //  not a run-length, val combination
      if (tmp < 10)
      {
          //  convert to negative number (coded that way to save space)
        if ((tmp == 2) && check_neg)
          tmp = -1 ;
        info[loc] = tmp ;
          //  for data like x positions, each value cumulative
        if (do_add)
          info[loc] += info[loc-1] ;
        loc++ ;
      }
      else
      {
        ctr = tmp / 10 ;	//  run length
        val = tmp % 10 ;	//  value
          // convert to negative number
        if ((val == 2) && check_neg)
          val = -1 ;
          //  decompress run
        for (i = ctr ; i > 0 ; i--, loc++)
        {
          info[loc] = val ;
          if (do_add)
            info[loc] += info[loc-1] ;
        }
      }
    }
  }


  int iget()
  {
	String wbuf ;
	int is_comment ;
	int is_eol ;
	int is_space ;
	int done = 2 ;

            // skip to beginning of next integer
        while (done != 1)
        {
	    done = 1 ;
            is_comment = fstr.indexOf('#',startw) ;
            while (is_comment == startw)
	    {
	        done = 0 ;
                startw = fstr.indexOf('\n',startw) + 1 ;
                is_comment = fstr.indexOf('#',startw) ;
	    }
            if (is_pc)
            {
                is_eol = fstr.indexOf('\r',startw) ;
                while (is_eol == startw)
	        {
	            done = 0 ;
	            startw += 2 ;
                    is_eol = fstr.indexOf('\r',startw) ;
	        }
            }
            else
            {
                is_eol = fstr.indexOf('\n',startw) ;
                while (is_eol == startw)
	        {
	            done = 0 ;
	            startw++ ;
                    is_eol = fstr.indexOf('\n',startw) ;
	        }
            }
        }
        if (is_pc)
        {
	    is_eol = fstr.indexOf('\r',startw) ;
        }
        else
        {
	    is_eol = fstr.indexOf('\n',startw) ;
        }
        is_space = fstr.indexOf(' ',startw) ;
	    //  find end of integer
	endw = (is_eol < is_space) ? is_eol : is_space ;
        wbuf = fstr.substring(startw,endw) ;
        if (is_pc)
        {
	    startw = (is_eol > is_space) ? (is_space+1) : (is_eol+2) ;
        }
        else
        {
	    startw = (is_eol > is_space) ? (is_space+1) : (is_eol+1) ;
        }
        //  System.out.println("thinks getting " + wbuf + " before parseInt");
        return(Integer.parseInt(wbuf)) ;
  }

  String cget(int j)
  {
    return(fstr.substring(startw+(j-2),startw+(j-1))) ;
  }

  void advance()
  {
    startw = fstr.indexOf('\n',startw)+1 ;
  }
}
