import java.awt.*;
import java.awt.event.*;
import java.lang.Thread ;

/**
 * Animation of the BART schedule
 * 
 *
 * @author David W. Robertson
 * @Lawrence Berkeley National Laboratory
 *
 * Last updated:  March 24, 2000

 * Bay Area Rapid Transit (BART) operates in the San Francisco
 * bay area.  Its schedules are widely disseminated; hundreds of
 * thousands of people ride every day.  I thought it would be
 * interesting to go from these lines of numbers to an "infrastructure
 * visualization" that gives an overlying view of the system's operation.
 *
 * A "line" refers to one direction of one of the major groupings of the
 * BART schedule, e.g. Bay Point to Colma, or Colma to Bay Point.
 * Each major grouping is color coded.  A "run" refers to one printed line
 * in the schedule.
 
 * This is a real-life application that shows the possibilities for animating
 * all sorts of schedules, not just BART.
 *
 * I'm working on making the code more object-oriented.  Ideally these
 * classes would serve as a schedule framework.
 */


public class BART extends java.applet.Applet implements ItemListener, ActionListener
{
  SchedManager schedManager ;
  Frame frame = null;
  int timefac ;		//  maximum time compression factor
  int train_size ;	//  train size control

  Panel img_panel = new Panel() ;
  MediaTracker tracker ;
  Panel contrls = new Panel() ;
//  Panel sched_panel = new Panel();
  Panel time_set_panel = new Panel();
  Panel mode_panel = new Panel();

//  CheckboxGroup sched_group = new CheckboxGroup() ;
//  Checkbox weekday_box = new Checkbox("Weekday",sched_group,true) ;
//  Checkbox saturday_box = new Checkbox("Saturday",sched_group,false) ;
//  Checkbox sunday_box = new Checkbox("Sunday",sched_group,false) ;
  final int WEEKDAY = 0 ;
  final int SATURDAY = 1 ;
  final int SUNDAY = 2 ;
  int which_schedule = WEEKDAY;

  private TextField timeEntry = new TextField(16);
  private Label timeLabel = new Label("Enter New Time:");

  Label mode_label = new Label("Speedup:") ;
  CheckboxGroup mode_group = new CheckboxGroup() ;
  Checkbox realtime_box = new Checkbox("None",mode_group,false) ;
  Checkbox simultime_box = new Checkbox("60:1",mode_group,true) ;

  public void init()
  {
    Component c = this;
    while (c.getParent() != null)
      c = c.getParent();
    if (c instanceof Frame)
    {
      frame = (Frame)c;
      String attr ;
      Color drk_red = new Color(160,0,0) ;

        // get maximum time speed-up attribute
      attr = getParameter("timefac") ;
      timefac = Integer.parseInt(attr) ;
        // if illegal value, use default
      if ((timefac < 1) || (timefac > 360))
        timefac = 360 ;
          // get train size attribute
      attr = getParameter("trsize") ;
      train_size = Integer.parseInt(attr) ;
        // if illegal value, use default
      if ((train_size < 6) || (train_size > 16))
        train_size = 8 ;
          // height bigger because image otherwise cropped
      img_panel.setSize(800,654) ;
      img_panel.setLayout(new BorderLayout()) ;
      contrls.setLayout(new BorderLayout());
          //  add control checkboxes
      // weekday_box.setForeground(Color.blue) ;
      // saturday_box.setForeground(Color.blue) ;
      // sunday_box.setForeground(Color.blue) ;
      // sched_panel.add(weekday_box) ;
      // sched_panel.add(saturday_box) ;
      // sched_panel.add(sunday_box) ;
      // contrls.add("Center",sched_panel);
      mode_panel.add(mode_label) ;
      realtime_box.setForeground(drk_red) ;
      simultime_box.setForeground(Color.black) ;
      mode_panel.add(realtime_box) ;
      mode_panel.add(simultime_box) ;
      contrls.add("East",mode_panel);
      time_set_panel.add(timeLabel);
      time_set_panel.add(timeEntry);
      contrls.add("West",time_set_panel);
        // create main drawing area and schedule manager
      schedManager = new SchedManager(this) ;
      schedManager.setSize(800,624) ;
      timeEntry.addActionListener(this);
      realtime_box.addItemListener(this);
      simultime_box.addItemListener(this);
      img_panel.add("Center",schedManager) ;
      img_panel.add("North",contrls) ;
      img_panel.setVisible(true) ;
      add(img_panel) ;
    }
  }


    // start train schedule from beginning
  public void start()
  {
    schedManager.start_it() ;
  }

    // stop train thread
  public void stop()
  {
    schedManager.stop_threads() ;
  }


    // wait for image to completely arrive
  void do_wait(Image img)
  {
    tracker = new MediaTracker(this) ;
    tracker.addImage(img,0) ;
    try { tracker.waitForID(0) ; }
    catch (InterruptedException e) {}
  }


  public void actionPerformed (ActionEvent evt)
  {
    String action_str;
    boolean isLegal;

    if (evt.getSource() instanceof TextField)
    {
      TextField tf = (TextField)evt.getSource();
      System.gc();
      isLegal = schedManager.changeTime(evt.getActionCommand());
      if (!isLegal)
        tf.setText("illegal entry");
      else
      {
        tf.setText("");
        if (schedManager.trainThread == null)
          schedManager.start_it();
      }
    }
    return;
  }


  public void itemStateChanged(ItemEvent evt)
  {
    Object which = evt.getItem();

    if ("None".equals(which))
        schedManager.real_mode(true) ;
    else
        schedManager.real_mode(false) ;
  }
}
